<?php

if (! class_exists('EG_Forms_105')) {
	require('lib/eg-forms.inc.php');
}

if (! class_exists('EG_Series_Admin')) {

	/**
	 * Class EG_Series_Admin
	 *
	 * Implement the admin interface of plugin EG-Series
	 *
	 */
	class EG_Series_Admin extends EG_Plugin_112 {

		var $series_data;
		var $edit_posts_pages = array('post.php', 'post-new.php', 'page.php', 'page-new.php');

		/**
		 * plugins_loaded
		 *
		 * Create the series data object to manage series data
		 *
		 * @param 	none
		 * @return 	none
		 */
		function plugins_loaded() {

			parent::plugins_loaded();

			// Add page for mass posts editing
			$this->add_page('posts', 				/* page type: post, page, option, tool 	*/
							'EG-Series Posts', 		/* Page title 							*/
							'EG-Series Posts', 		/* Menu title 							*/
							'edit_pages', 			/* Access level / capability			*/
							'egs_post_edit',		/* file 								*/
							'bulk_posts',			/* function								*/
							FALSE,
							'bulk_posts_load_scripts');

			// Add page for series editing: rename series
			$this->add_page('posts', 				/* page type: post, page, option, tool 	*/
							'EG-Series Edit',		/* Page title 							*/
							'EG-Series Edit',		/* Menu title 							*/
							'edit_pages', 			/* Access level / capability			*/
							'egs_series_edit',	 	/* file 								*/
							'edit_series');			/* function								*/

			// Add plugin options page
			$this->add_page('options', 				/* page type: post, page, option, tool 	*/
							'EG-Series options',	/* Page title 							*/
							'EG-Series',			/* Menu title 							*/
							'edit_pages', 			/* Access level / capability			*/
							'egs_series_options',	/* file 								*/
							'options_page');		/* function								*/
		} /* End of plugins_loaded */

		/**
		 * init
		 *
		 *
		 *
		 * @param 	none
		 * @return 	none
		 */
		function admin_init() {
			global $pagenow;

			parent::admin_init();

			$this->series_data = & EG_Series_Data::get_instance($this->textdomain);

			if ($this->options['date_format'] == '') {
				$this->options['date_format'] = get_option('date_format');
				update_option($this->options_entry, $this->options);
			}
			wp_register_script('eg-series', $this->plugin_url.'lib/eg-series.js', array('jquery-ui-sortable'), '1.0.0');
			wp_localize_script('eg-series', 'egSeriesSetup', array(
					'ajax_url'		 => admin_url('admin-ajax.php'),
					'UpdateSeries' 	 => 'eg_series_update',
					'GetPosts'       => 'eg_series_get_posts_list',
					'GetPostsNumber' => 'eg_series_get_posts_number',
					'UpdateOk'       => __('Series and posts updated successfully !', $this->textdomain),
					'UpdateError'    => __('A failure occurred.<br />Please reload page before continuing', $this->textdomain),
					'UpdateSecurity' => __('Suspicious security issue. Operation cancelled !', $this->textdomain)
					/* 'nonce' 		 => wp_create_nonce( 'egseries-ajax' ), */
			) );

			// Add columns to the post list (menu Manage)
			if ($this->options['display_columns']) {
				add_filter('manage_posts_columns', array(&$this, 'manage_col_def') );
				add_action('manage_posts_custom_column', array(&$this, 'manage_col'), 10, 2 );
			}
			if (! version_compare($wp_version, '2.8', '<') )
				add_filter('screen_layout_columns', array(&$this, 'on_screen_layout_columns'), 10, 2);

			add_action('wp_ajax_eg_series_update',           array(&$this, 'bulk_posts_series_update') );
			add_action('wp_ajax_eg_series_get_posts_list',   array(&$this, 'metabox_posts_list')       );
			add_action('wp_ajax_eg_series_get_posts_number', array(&$this, 'metabox_posts_number')     );

			if ($this->options['use_metabox'] && function_exists( 'add_meta_box' ) &&
				in_array($pagenow, $this->edit_posts_pages) ) {

				add_action('load-'.$pagenow, array(&$this, 'load_edit_post_pages'));
			}

		} /* End of init */

		/**
		 * admin_menu
		 *
		 * Add metabox and save post action
		 *
		 * @param	none
		 * @return 	none
		 */
		function admin_menu() {
			global $pagenow;

			parent::admin_menu();

			if ($this->options['use_metabox'] && function_exists( 'add_meta_box' ) &&
				in_array($pagenow, $this->edit_posts_pages) ) {

				add_meta_box( 'eg-series-metabox', __( 'EG-Series', $this->textdomain ),
							array(&$this, 'display_metabox'), 'post', 'normal', 'high' );

				add_meta_box( 'eg-series-metabox', __( 'EG-Series', $this->textdomain ),
							array(&$this, 'display_metabox'), 'page', 'normal', 'high' );

				add_action( 'save_post', array(&$this, 'save_metabox'), 10, 2);
			}
		} // End of admin_menu

		/**
		 * load_edit_post_pages
		 *
		 * Load script required for metaboxes
		 *
		 * @param	none
		 * @return 	none
		 */
		function load_edit_post_pages() {
			wp_enqueue_script('eg-series');
		} // End of load_edit_post_pages

		/**
		 * display_metabox
		 *
		 * Display series meta box
		 *
		 * @param	object	$post		post or page currently edited
		 * @param	array	$args		other arguments
		 * @return 	none
		 */
		function display_metabox($post, $args) {

			$meta_key      = $this->options['custom_field_name'];
			$series_list   = $this->series_data->get_series_details($meta_key, 'menu_order', 'ASC', TRUE);
			$current_serie = FALSE;
			if ( isset($post->ID) )
				$current_serie = get_post_meta($post->ID, $meta_key, TRUE);

			$select_series = '';
			if ($series_list !== FALSE &&  sizeof($series_list)>0) {
				$select_series = '<select id="egseries-select" name="egseries-select" onchange="EGSerieListPost(); EGSeriePostNumber();"><option value="'.EG_SERIES_NO_SERIE_ID.'">'.__('Select a serie', $this->textdomain).'</option>';
				foreach ($series_list as $serie => $posts) {
					$selected = ($serie == $current_serie?'selected':'');
					$select_series .= '<option value="'.$serie.'" '.$selected.'>'.htmlspecialchars($serie).'</option>';
				}
				$select_series .= '</select>';

				$order_list = '<select id="egseries-order" name="egseries-order">';
				if (isset($current_serie) && $current_serie != EG_SERIES_NO_SERIE_ID && $current_serie!='') {
					$number_of_posts = sizeof($series_list[$current_serie]);
					for ($order=1; $order <= $number_of_posts; $order++) {
						$selected = ($order==$post->menu_order?'selected':'');
						$order_list .= '<option value="'.$order.'" '.$selected.'>'.$order.'</option>';
					}
				}
				$order_list .= '</select>';
			}
?>
			<div id="egseries-stuff">
				<form id="egseries-form" name="egseries-form" method="POST" action="#">
				<input type="hidden" id="egseries-current-order" name="egseries-current-order"value="<?php echo $post->menu_order; ?>" />
				<input type="hidden" id="egseries-current-serie" name="egseries-current-serie"value="<?php echo ($current_serie?$current_serie:''); ?>" />				
				<table id="egseries-box">
					<tbody>
						<tr>
							<td class="left">
<?php
			if ($select_series != '') {
?>
								<div id="egseries_existing">
									<?php echo $select_series . $order_list; ?>
								</div>

								<input style="display: none;" class="hide-if-js" id="egseries-input" name="egseries-input"  value="" type="text"><br />
								<a href="#" class="hide-if-no-js" onclick="jQuery('#egseries-input, #egseries_existing, #egseries-enternew, #egseries-cancelnew, #egseries-posts-list').toggle(); return false;">
									<span style="display: inline;" id="egseries-enternew"><? _e('Enter new', $this->textdomain); ?></span>
									<span style="display: none;" id="egseries-cancelnew" class="hidden"><? _e('Cancel', $this->textdomain); ?></span>
								</a>
<?php
			}
			else {
								_e('Enter the name of a new serie: ', $this->textdomain);
?>
								<input id="egseries-input" name="egseries-input"  value="" type="text"><br />
<?php
			}
?>
						</td>
						<td>
							<div id="egseries-posts-list" style="display: inline;">
							<?php
								if ( isset($current_serie) )
									echo $this->list_serie_posts($meta_key,$current_serie);
								else
									echo '&nbsp;';
							?>
							</div>
						</tr>
					</tbody>
				</table>
				</form>
			</div>
<?php
		} // End of display_metabox

		/**
		 * save_metabox
		 *
		 * Save selected serie
		 *
		 * @param 	int		$post_id		id of the currently edited post
		 * @param	object	$post			post or page currently edited
		 * @return 	none
		 */
		function save_metabox($post_id, $post) {

			// Don't save serie for revision
			if ($post->post_type == 'post' || $post->post_type == 'page') {

				$meta_key       = $this->options['custom_field_name'];
				$previous_value = get_post_meta($post->ID, $meta_key, TRUE);

				$series_list = $this->series_data->get_series_details($meta_key);

				$eg_series_new    = $_POST['egseries-input'];
				$eg_series_select = $_POST['egseries-select'];
				$eg_series_order  = $_POST['egseries-order'];
				$previous_order   = $_POST['egseries-current-order'];

				// Case of new serie
				if ($eg_series_new != '') {
					// New serie entered already exists in the list
					if (isset($series_list[$eg_series_new])) {
						$eg_series_select = $eg_series_new;
						$eg_series_order  = 0;
					}
					else {
						// New serie is really a new serie
						$eg_series_select = '';
						if ($previous_value == '') {
							// The current post is not yet associated
							add_post_meta($post_id, $meta_key, $eg_series_new, true);

						}
						else {
							// Post is already in another serie
							update_post_meta($post_id, $meta_key, $eg_series_new);

							// re-order the previous serie
							foreach ($series_list[$previous_value] as $item) {
								if ($item->menu_order > $previous_order) {
									$this->update_post_order($item->ID, $item->menu_order-1);
								}
							}
						} // End of "post already exists in another serie
						// Add in first position
						$this->update_post_order($post_id, 1);
					} // End of it's really a new serie
				} // End of new serie

				// Case of list chosen in the list
				if ($eg_series_select != '') {
					if ($eg_series_select != $previous_value) {
						if ($previous_value != '') {
							// Remove from the serie
							delete_post_meta($post_id, $meta_key, $previous_value);

							// re-order the previous serie
							foreach ($series_list[$previous_value] as $item) {
								if ($item->menu_order > $previous_order) {
									$this->update_post_order($item->ID, $item->menu_order-1);
								}
							}
							$this->update_post_order($post_id, 0);
						}

						if ($eg_series_select != EG_SERIES_NO_SERIE_ID) {
							// Add post into the new serie
							add_post_meta($post_id, $meta_key, $eg_series_select);

							// Re-order the current serie
							if ($eg_series_order < 1)
								$eg_series_order = sizeof($series_list[$eg_series_select])+1;
							if ($eg_series_order < sizeof($series_list[$eg_series_select])) {
								foreach ($series_list[$eg_series_select] as $item) {
									if ($item->menu_order > $eg_series_order) {
										$this->update_post_order($item->ID, $item->menu_order-1);
									}
								}
							}
							$this->update_post_order($post_id, $eg_series_order);
						} // End of existing serie selected
					} // End of current_value differant than previous_value
					else {
						if ($eg_series_order != $previous_order) {
							if ($eg_series_order < $previous_order) {
								foreach ($series_list[$eg_series_select] as $item) {
									if ($item->menu_order >= $eg_series_order && $item->menu_order < $previous_order) {
										$this->update_post_order($item->ID, $item->menu_order+1);
									}
								}
							} // End of $eg_series_order < $previous_order
							else {
								foreach ($series_list[$eg_series_select] as $item) {
									if ($item->menu_order > $previous_order && $item->menu_order <= $eg_series_order ) {
										$this->update_post_order($item->ID, $item->menu_order-1);
									}
								}
							} // End of $eg_series_order > $previous_order
							$this->update_post_order($post_id, $eg_series_order);
						} // Change post order in the same serie
					} // Just re-order the serie
				} // End of select existing serie or no_serie
			} // End of post_type = post or page
		} // End of save_metabox

		/**
		 * metabox_posts_list
		 *
		 * Query database, and return list of posts for a specific serie
		 *
		 * @param	none
		 * @return 	none
		 */
		function metabox_posts_list() {

			die ( $this->list_serie_posts($this->options['custom_field_name'], $_POST['serie']) );

/*			$sql = $wpdb->prepare('SELECT DISTINCT ID,menu_order, post_title FROM '.$wpdb->posts.', '.$wpdb->postmeta.' WHERE meta_key=%s AND meta_value=%s AND ID=post_id ORDER BY menu_order ASC', $meta_key, $serie_name);
			$results = $wpdb->get_results($sql);
			$output = '<ul>'."\n";
			foreach ($results as $result) {
				$output .= '<li>'.$result->menu_order.'. '.htmlspecialchars($result->post_title).'</li>'."\n";
			}
			$output .= '</ul>'."\n";
			die( $output );
*/
		} // End of metabox_posts_list

		/**
		 * list_serie_posts
		 *
		 * Display posts of a specified serie
		 *
		 * @param 	string		$meta_key		mta_key used for series
		 * @param	string		$serie_name		serie we want to display posts
		 * @return 	none
		 */
		function list_serie_posts($meta_key, $serie_name) {

			$series_list = $this->series_data->get_series_details($meta_key, 'menu_order', 'ASC');
			$output = '';
			if ( $series_list !== FALSE && sizeof($series_list)>0 &&
				$serie_name != EG_SERIES_NO_SERIE_ID && $serie_name != '') {
				$output = '<ul>';
				foreach ($series_list[$serie_name] as $post) {
					$output .= '<li>'.$post->menu_order.'. '.$post->post_title.'</li>';
				}
				$output .= '</ul>';
			}
			return ($output);
		} // End of list_serie_posts

		/**
		 * metabox_posts_number
		 *
		 * Query database, and return the number of posts for a specific serie
		 *
		 * @param	none
		 * @return 	none
		 */
		function metabox_posts_number() {
			$series_list = $this->series_data->get_series_details($this->options['custom_field_name']);
			// Use "string" because die doesn't output anything if the value is integer !!! 
			die( (string) sizeof($series_list[$_POST['serie']]) );
		} // End of metabox_posts_number

		/**
		 * on_screen_layout_columns
		 *
		 * Add metabox and save post action
		 *
		 * @param	none
		 * @return 	none
		 */

		//for WordPress 2.8 we have to tell, that we support 2 columns !
		function on_screen_layout_columns($columns, $screen) {
			if ($screen == $this->pagehook) {
				$columns[$this->pagehook] = 2;
			}
			return $columns;
		} // End of on_screen_layout_columns

		/**
		 * manage_col_def
		 *
		 * Add columns to the admin page: manage post
		 *
		 * @package EG-Series
		 * @param array	$cols		list of columns header
		 * @return array	$cols
		 */
		function manage_col_def($cols) {
			$cols['serie'] = __('Serie', $this->textdomain);
			return $cols;
		} /* End of manage_col_def */

		/**
		 * manage_col
		 *
		 * Complete the column 'serie'
		 *
		 * @package EG-Series
		 * @param string	$column_name	name of the column
		 * @param int		$id				post id
		 * @return none
		 */
		function manage_col($column_name, $id) {
			if ($column_name == 'serie') echo htmlspecialchars(get_post_meta($id, $this->options['custom_field_name'], true));
			else echo '&nbsp;';
		} /* End of manage_col */

		/**
		 * add_form
		 *
		 * create form
		 *
		 * @package EG-Series
		 *
		 * @param none
		 * @return none
		 */
		function add_form() {
			$this->options_form = new EG_Forms_105('EG-Series Options', '', '', $this->textdomain, '', 'icon-options-general', 'egs_options', 'mailto:'.EG_SERIES_AUTHOR_EMAIL);
			$form = & $this->options_form;

			$id_section = $form->add_section('Interface');
			$id_group   = $form->add_group($id_section, 'Post management list');
			$form->add_field($id_section, $id_group, 'checkbox', 'Add column \'Series\' to the post management page', 'display_columns');
			$id_group   = $form->add_group($id_section, 'Post editor page');
			$form->add_field($id_section, $id_group, 'checkbox', 'Use metabox to enter the series, rather than enter custom fields manually.', 'use_metabox', '', '', 'Plugin can display a metabox in the post editor page, to make easy the serie choice or creation. But you can hide this metabox, and enter manually series name if the custom field');

			$id_section = $form->add_section('Shortcodes options', 'You can choose keywords you want use for shortcodes.<br />Change them just after the plugin installation. In other case, all EG-Series shortcodes already used in posts, won\'t work properly after the change.');
			$id_group   = $form->add_group($id_section, 'Shortcode to list series');
			$form->add_field($id_section, $id_group, 'text', 'Shortcode to list series', 'shortcode_list_series', '', '', 'Use to list series in a post');
			$id_group   = $form->add_group($id_section, 'Shortcode to list posts');
			$form->add_field($id_section, $id_group, 'text', 'Shortcode to list posts', 'shortcode_list_posts', '', '', 'Use to list all posts of the same serie than the current post');
			$id_group   = $form->add_group($id_section, 'Date Format');
			$form->add_field($id_section, $id_group, 'text', 'Date format: ', 'date_format', '', '('.date_i18n($this->options['date_format']).')',
											'Use PHP date() parameters to configure the date:<br />d&nbsp;&nbsp;&nbsp;&nbsp;Day of the month, 2 digits with leading zeros<br />D&nbsp;&nbsp;&nbsp;&nbsp;A textual representation of a day, three letters<br />j&nbsp;&nbsp;&nbsp;&nbsp;Day of the month without leading zeros<br />S&nbsp;&nbsp;&nbsp;&nbsp;English ordinal suffix for the day of the month, 2 characters<br />F&nbsp;&nbsp;&nbsp;&nbsp;A full textual representation of a month, such as January or March<br />m&nbsp;&nbsp;&nbsp;&nbsp;Numeric representation of a month, with leading zeros<br />M&nbsp;&nbsp;&nbsp;&nbsp;A short textual representation of a month, three letters<br />n&nbsp;&nbsp;&nbsp;&nbsp;Numeric representation of a month, without leading zeros<br />Y&nbsp;&nbsp;&nbsp;&nbsp;A full numeric representation of a year, 4 digits<br />y&nbsp;&nbsp;&nbsp;&nbsp;A two digit representation of a year');

			$id_group   = $form->add_group($id_section, 'Auto shortcode');
			$form->add_field($id_section, $id_group, 'select', 'Automatically add the list of posts that are in the same serie than the current displayed post: ', 'shortcode_auto', '', '', '', '', 'regular', array( 0 => 'Not activated', 2 => 'At the end')); // 1 => 'At the beginning'
			$form->add_field($id_section, $id_group, 'checkbox', 'Activate automatic shortcode on the following cases', 'shortcode_auto_where', '', '', '', '', 'regular', array( 'home' => 'Homepage,', 'post' => 'Post,', 'page' => 'Page,', 'index' => 'Lists of posts.'));

			$form->add_field($id_section, $id_group, 'text', 'Title of the list: ', 'shortcode_auto_title');
			$form->add_field($id_section, $id_group, 'text', 'HTML Tag for title: ', 'shortcode_auto_title_tag');
			$form->add_field($id_section, $id_group, 'select', 'List type: ', 'shortcode_auto_listtype', '', '', '', '', 'regular', array( 'select' => 'Select', 'ul' => 'Simple list', 'ol' => 'Ordered list') );
			$form->add_field($id_section, $id_group, 'select', 'Order by: ', 'shortcode_auto_orderby', '', '', '', '', 'regular', array( 'date' => 'Date', 'title' => 'Title', 'user_order' => 'Order') );
			$form->add_field($id_section, $id_group, 'select', 'Order: ', 'shortcode_auto_order', '', '', '', '', 'regular', array( 'ASC' => 'Ascending', 'DESC' => 'Descending') );
			$form->add_field($id_section, $id_group, 'checkbox', 'Display dates if checked', 'shortcode_auto_show_date', 'Display dates? ', '', '', '', 'regular' );
			$form->add_field($id_section, $id_group, 'checkbox', 'Do you want that auto shortcode options become the default options for the TinyMCE EG-Series Editor?', 'shortcode_auto_default_opts', 'Default options? ', '', '', '', 'regular' );

			$id_section = $form->add_section('Custom fields options', 'You can choose the custom field you want to use to implement series. You can change this value at any time: the field will be change automatically in the WordPress database.');
			$id_group   = $form->add_group($id_section, 'Choose the custom field to use');
			$form->add_field($id_section, $id_group, 'text', 'Custom field to use', 'custom_field_name', '', '', 'If you already have a field used for series.');

			$id_section = $form->add_section('Uninstallation', '', 'Be careful: the deletion can\'t be cancelled');
			$id_group   = $form->add_group($id_section, 'Options');
			$form->add_field($id_section, $id_group, 'checkbox', 'Check if you want to delete options while plugin uninstallation', 'uninstall_del_option', '', '', '', '', 'regular' );
			$id_group   = $form->add_group($id_section, 'Custom fields');
			$form->add_field($id_section, $id_group, 'checkbox', 'Check if you want to delete custom fields', 'uninstall_del_custom_field', '', '', '', '', 'regular' );

			$form->add_button('submit', 'eg_series_options_submit', 'Save changes');
			$form->add_button('reset', 'eg_series_options_reset', 'Cancel changes');
			$form->add_button('submit', 'eg_series_options_reset', 'Reset to defaults', 'reset_to_defaults');

		} /* End of add_form */


		/**
		 * options_page
		 *
		 * Display the options page
		 *
		 * @param 	none
		 * @return 	none
		 */
		function options_page() {
			global $EG_SERIES_DEFAULT_OPTIONS;

			$this->add_form();

			$results = $this->options_form->get_form_values($this->options, $EG_SERIES_DEFAULT_OPTIONS, $this->options_entry);
			if ($results)
				$this->options = $results;

			$this->options_form->display_form($this->options);

		} /* options_page */

		/**
		 * install_upgrade
		 *
		 * Install or upgrade options, tables, ...
		 *
		 * @param 	none
		 * @return 	none
		 */
		function install_upgrade() {
			global $wpdb;

			$previous_options = parent::install_upgrade();

			if (version_compare($previous_options['version'], $this->plugin_version, '<')) {
				if (version_compare($previous_options['version'], '1.3.0', '<')) {

					$meta_key = $this->options['custom_field_name'];
					if (!isset($this->series_data))
						$this->series_data = & EG_Series_Data::get_instance($this->textdomain);

					$series = $this->series_data->get_series_details($meta_key, 'date', 'DESC');

					foreach ($series as $serie => $posts_list) {
						$order = 1;
						foreach ($posts_list as $post) {
							$post->menu_order = $order++;
							$this->update_post_order($post->ID, $post->menu_order);
						} // End of foreach posts
					} // End of foreach series
				} // End of version 1.2.0
			} // End current version upper than the previous
		} // End of install_upgrade

		/**
		 * bulk_posts_load_scripts
		 *
		 * Load scripts for series bulk editor
		 *
		 * @param 	none
		 * @return 	none
		 */
		function bulk_posts_load_scripts() {
			wp_enqueue_script('postbox');
			wp_enqueue_script('jquery-ui-sortable');
			wp_enqueue_script('eg-series');
		} // End of bulk_posts_load_scripts

		/**
		 * bulk_posts
		 *
		 * Allow to edit series field for all posts
		 *
		 * Thanks to Armaury Balmer and its simple tags plugin that was a good source of tips for these 'mass edit' function.
		 *
		 * @param 	none
		 * @return 	none
		 */
		function bulk_posts() {

			$hook = current($this->hooks['display'][__FUNCTION__]);
			add_meta_box('eg-series-posts-list', __('Available posts', $this->textdomain), array(&$this, 'bulk_posts_display_posts'), $hook, 'normal', 'core');
			// Get the list of series and store it in an array
			$series_list = $this->series_data->get_series_details($this->options['custom_field_name'], 'menu_order', 'ASC', TRUE);
			$id = 1;
			$series_id = array();
			if ($series_list !==FALSE && sizeof($series_list) > 0) {
				foreach ($series_list as $serie => $posts) {
					add_meta_box('seriebox-'.$id, $serie, array(&$this, 'bulk_posts_display_series'), $hook, 'side', 'core');
					$series_id['seriebox-'.$id] = $serie;
					$id++;
				}
			}
			if (get_option(EG_SERIES_IDS) === FALSE)
				add_option(EG_SERIES_IDS, $series_id);
			else
				update_option(EG_SERIES_IDS, $series_id);

?>
			<div id="icon-tools" class="icon32"><br /></div>
			<div class="wrap">
				<h2><?php _e('EG-Series Bulk Edit', $this->textdomain); ?></h2>
				<div id="poststuff" class="metabox-holder has-right-sidebar">
					<div id="side-info-column" class="inner-sidebar">
						<?php do_meta_boxes($hook, 'side', $series_id); ?>
					</div>
					<div id="post-body" class="has-sidebar">
						<div id="post-body-content" class="has-sidebar-content">
							<?php do_meta_boxes($hook, 'normal', FALSE); ?>
						</div>
					</div>
					<br class="clear"/>
				</div>
			</div>
			<script type="text/javascript">
				//<![CDATA[
				jQuery(document).ready( function($) {
					postboxes.add_postbox_toggles('<?php echo $hook; ?>');
				});
				//]]>
			</script>
<?php
		} /* End of bulk_posts */

		/**
		 * bulk_posts_display_posts
		 *
		 * Display posts list and filters
		 *
		 * @param 	none
		 * @return 	none
		 */
		function bulk_posts_display_posts($data) {
			global $wpdb, $wp_locale;

			/* ---------------------------
			    Default parameters
			------------------------------ */
			$requested_status   = 'publish';
			$requested_category = '';
			$order              = 'DESC';
			$orderby            = 'date';
			$post_per_page      = 25;
			$requested_year     = 0;
			$requested_month    = 0;
			$requested_date     = 0;

			// Get date parameter
			$status_title = array(
				'all'     => __('All posts', $this->textdomain),
				'future'  => __('Future',    $this->textdomain),
				'publish' => __('Published', $this->textdomain),
				'draft'   => __('Draft',     $this->textdomain),
				'pending' => __('Pending',   $this->textdomain)
			);

			/* --------------------------------------------
				Collect parameters
			   -------------------------------------------- */

			// Manage the number of the page requested
			if ( !isset( $_GET['paged'] ) ) {
				$_GET['paged'] = 1;
			}
			$paged = $_GET['paged'];

			if (isset($_GET['post_status']) && array_key_exists($_GET['post_status'],$status_title ))
				$requested_status = $_GET['post_status'];

			// Get date parameter
			if (isset($_GET['date']) && $_GET['date']!=0) {
				$requested_date  = $_GET['date'];
				$requested_year  = substr($_GET['date'], 0, 4);
				$requested_month = substr($_GET['date'], 5);
			}

			// Get category parameter
			if (isset($_GET['cat']))
				$requested_category = (int) $_GET['cat'];

			/* -----------------------------------------------------
			   --- Query database
			   ----------------------------------------------------- */
/*
			$sql = 'SELECT ID FROM '.$wpdb->prefix.'posts,'.$wpdb->prefix.'postmeta WHERE meta_key="'.$this->options['custom_field_name'].'" AND ID=post_id';
			$results = $wpdb->get_results($sql);

			foreach ($results as $result) {
				$exclude_list[] = (int) $result->ID;
			}
			unset($results);
*/
			$exclude_list = array();
			$series_list  = $this->series_data->get_series_details($this->options['custom_field_name']);
			if ( $series_list !==FALSE && sizeof($series_list)>0 ) {
				foreach ($series_list as $serie => $posts) {
					foreach ($posts as $post) {
						$exclude_list[] = $post->ID;
					}
				}
			}

			query_posts( array('post_type' 	=> 'any',
								'post_status' 		=> $requested_status,
								'cat'				=> $requested_category,
								'year'				=> $requested_year,
								'monthnum'			=> $requested_month,
								'post__not_in'		=> $exclude_list,
								'posts_per_page' 	=> $post_per_page,
								'paged'				=> $paged,
								'orderby'			=> $orderby,
								'order'				=> $order)
			);

			/* -----------------------------------------------------
			   --- Posts status selector
			   ----------------------------------------------------- */
			$num_posts   = wp_count_posts('post', 'readable');
			$total_posts = array_sum( (array) $num_posts );
			$url_page    = admin_url('edit.php?page=egs_post_edit');
			$num_posts   = array_merge( array('all' => $total_posts), (array) $num_posts);
			foreach ($num_posts as $status => $number) {
				if (isset($status_title[$status])) {
					$class = ($requested_status == $status ? ' class="current"' : '');
					$filter_post_status[] = '<li>'.
						'<a href="'.$url_page.($status=='all'?'':'&post_status='.$status).'" '.$class.'>'.
						$status_title[$status].' <span class="count">'.($number<0?'':'('.$number.')').'</span>'.
						'</a></li>';
				}
			}

			/* -----------------------------------------------------
			   --- Category selector
			   ----------------------------------------------------- */
			$filter_category = wp_dropdown_categories(array(
						'show_option_all' => __('View all categories', $this->textdomain),
				        'hide_empty'      => 1,
						'hierarchical'    => 1,
						'show_count'      => 1,
						'orderby'         => 'name',
						'selected'        => $requested_category,
						'echo'			  => 0)
					);

			/* -----------------------------------------------------
			   --- Date / Archives Selector
			   ----------------------------------------------------- */
			$results = wp_cache_get( 'archives', EG_SERIES_CACHE_GROUP );
			if (! $results) {
				$results = $wpdb->get_results( 'SELECT DISTINCT YEAR(post_date) AS yyear, MONTH(post_date) AS mmonth FROM '.$wpdb->posts.' WHERE post_type = "post" ORDER BY post_date DESC' );
				wp_cache_set('archives', $results, EG_SERIES_CACHE_GROUP, EG_SERIES_CACHE_EXPIRATION);
			}
			$month_count = count($results);
			if ( $month_count && !( 1 == $month_count && 0 == $results[0]->mmonth ) ) {
				$filter_archive = '<select name="date">'.
					 '<option value="0">'.__('Show all dates', $this->textdomain).'</option>';
				foreach ($results as $arc_row) {
						$arc_row->mmonth = zeroise( $arc_row->mmonth, 2 );

						$default = ( $arc_row->yyear.$arc_row->mmonth == $requested_date ? ' selected="selected"': '');

						$filter_archive .= '<option'.$default.' value="'.$arc_row->yyear.$arc_row->mmonth.'">'.
							 $wp_locale->get_month($arc_row->mmonth) . ' '.$arc_row->yyear.
							 '</option>';
				}
				$filter_archive .='</select> ';
			}

			global $wp_query;
			$page_links = paginate_links( array(
				'base'		=> add_query_arg( 'paged', '%#%' ),
				'format' 	=> '',
				'prev_text' => __('&laquo;'),
				'next_text' => __('&raquo;'),
				'total'		=> $wp_query->max_num_pages,
				'current'	=> $_GET['paged']
			));

			if ( $page_links ) {
				$page_links_text = '<div class="tablenav-pages">'.
					sprintf( '<span class="displaying-num">' . __( 'Displaying %s&#8211;%s of %s', $this->textdomain ) . '</span>%s',
	number_format_i18n( ( $_GET['paged'] - 1 ) * $wp_query->query_vars['posts_per_page'] + 1 ),
	number_format_i18n( min( $_GET['paged'] * $wp_query->query_vars['posts_per_page'], $wp_query->found_posts ) ),
	number_format_i18n( $wp_query->found_posts ),
	$page_links).'</div>';
			}

			/* Query the list of the posts to display */
?>
		<p class="description"><?php _e('Drag posts from here to a serie on the right to include them in a serie. Drag posts back here remove them from series.', $this->textdomain); ?></p>
		<div id="ajaxquery">&nbsp;</div>
		<div id="mass_edit_filter">
			<form id="posts-filter" action="" method="get">
				<ul class="subsubsub">
					<?php echo implode(' | ', $filter_post_status); ?>
				</ul>
				<div class="tablenav">
					<input type="hidden" name="page" value="egs_post_edit" />
					<input type="hidden" name="post_status" value="<?php echo attribute_escape($requested_status); ?>" />
					<?php echo $filter_archive.' '.$filter_category; ?>
					<input type="submit" id="post-query-submit" value="<?php _e('Filter', $this->textdomain); ?>" class="button-secondary" />
						<?php echo $page_links_text; ?>
					<div class="clear"></div>
				</div>
			</form>
			<div class="clear"></div>
		</div>
		<ul id="posts-list" class="egseries-sortable">
<?php
			while (have_posts()) {
				the_post();
				echo '<li id="post-'.get_the_ID().'" >'.get_the_title().'</li>';
			}
?>
		</ul>
<?php
		} // End of display_post_list

		/**
		 * bulk_posts_display_series
		 *
		 * Display posts list and filters
		 *
		 * @param 	none
		 * @return 	none
		 */
		function bulk_posts_display_series($data, $box) {

			$series_list = $this->series_data->get_series_details($this->options['custom_field_name']);
			$serie_name  = $box['title'];
			$output      = '<ul id="'.$box['id'].'" class="eg-series egseries-sortable">';
			foreach ($series_list[$serie_name] as $post) {
				// $output .= '<li id="post-'.$post->ID.'"><a title="'.htmlspecialchars($post->post_title).'">'.wp_html_excerpt($post->post_title, 35).'</a></li>';
				$output .= '<li id="post-'.$post->ID.'">'.htmlspecialchars($post->post_title).'</li>';
			}
			$output .= '</ul>';

			echo $output;

		} // End of display_post_list

		/**
		 * bulk_posts_series_update
		 *
		 * Update series according request
		 *
		 * @param 	none
		 * @return 	none
		 */
		function bulk_posts_series_update() {
			global $wpdb;

			$error_code = 1;
			if (! isset($this->options))
				$this->options = get_option(EG_SERIES_OPTIONS_ENTRY);

			$serie_id    = $_POST['serie'];
			$post_order  = $_POST['post'];
			$series_list = get_option(EG_SERIES_IDS);

			if (! isset($series_list[$serie_id]))  {
				$error_code = 2;
			}
			else {
				foreach ($post_order as $key => $post) {
					$post_order[$key] = (int)$post;
				}
				$serie_name = $series_list[$serie_id];
				if (!isset($this->options))
					$this->options = get_option(EG_SERIES_OPTIONS_ENTRY);
				$meta_key = $this->options['custom_field_name'];

				// Get the current serie configuration
				$sql = $wpdb->prepare('SELECT DISTINCT ID FROM '.$wpdb->posts.', '.$wpdb->postmeta.' WHERE meta_key=%s AND meta_value=%s AND ID=post_id', $meta_key, $serie_name);
				$results = $wpdb->get_results($sql, ARRAY_N);
				foreach ($results as $result) {
					$current_posts[] = (int)$result[0];
				}
				unset($results);

				// Delete phase
				foreach ($current_posts as $post_id) {
					if (array_search($post_id, $post_order) === FALSE) {
						delete_post_meta($post_id, $meta_key, $serie_name);
						$this->update_post_order($post_id, 0);
					}
				} // End of deleting loop

				// Adding Phase
				foreach ($post_order as $post_id) {
					if (array_search($post_id, $current_posts) === FALSE) {
						add_post_meta($post_id, $meta_key, $serie_name);
					}
				} // End of adding loop

				// Re-order the list
				$order = 1;
				foreach ($post_order as $post) {
					$this->update_post_order($post, $order++);
				} // End of re-order loop

			} // End of no error
			die ($error_code);
		} // End of bulk_posts_series_update

		/**
		 * update_mass_serie
		 *
		 *  Get results from the form, and update the database.
		 *
		 * @param 	none
		 * @return 	none
		 */
		function update_mass_series() {

			// Detect if user click on the submit button
			$updated = 0;
			if (isset($_POST['eg_series_update_mass'])) {

				// check security
				check_admin_referer( 'eg_series_mass_edit' );

				/* Get the list of the serie per post */
				$current_series = $_POST['series'];
				$new_series     = $_POST['new_series'];
				$previous       = $_POST['previous_series'];
				$meta_key       = $this->options['custom_field_name'];

				foreach ($previous as $post_id => $previous_value) {

					$new_value = '';
					if ($new_series[$post_id] != '') {
						$new_value = $new_series[$post_id];
					}
					else {
						$current_serie = $current_series[$post_id];
						if ($current_serie != EG_SERIES_NO_SERIE_ID && $current_serie != '') {
							$new_value = $current_serie;
						}
					}
					if ($new_value != $previous_value) {
						$updated ++;
						if ($new_value == '') {
							delete_post_meta($post_id, $meta_key);
						}
						else {
							if ($previous_value == '') {
								add_post_meta($post_id, $meta_key, $new_value, TRUE);
							}
							else {
								update_post_meta ($post_id, $meta_key, $new_value);
							}
						}
						clean_post_cache($post_id);
					}
				} /* End for loop */

				if ($updated > 0) {
					$this->display_message(sprintf(__('%s posts were updated successfully !', $this->textdomain), $updated));
					$this->series_data->get_series_details($meta_key, FALSE, FALSE, TRUE);
				}
			} /* End of if $_POST */
			return ($update != 0);
		} /* End of update_mass_series */

		/**
		 * edit_series
		 *
		 * Edit series (allow to rename them)
		 *
		 * @param 	none
		 * @return 	none
		 */
		function edit_series() {
			global $wpdb;

			$meta_key = $this->options['custom_field_name'];

			// if user click on the submit button
			if (isset($_POST['eg_series_rename'])) {

				// check security
				check_admin_referer('eg_series_edit');

				// Get list of series
				$series = $_POST['series'];
				$updated = 0;
				if (isset($series) && sizeof($series)>0) {

					// For each serie, test if the name exist, and if the new name is different than the previous
					foreach ($series as $old_name => $new_name) {
						if ($new_name != '' && $new_name != $old_name) {
							// Count number of updates
							$updated++;
							// Update database
							$sql = $wpdb->prepare( 'UPDATE '.$wpdb->postmeta.' SET meta_value="%s" WHERE meta_key = "%s" AND meta_value = "%S"', $new_name, $meta_key, $old_name);
							$wpdb->get_query( $sql );
						}
					}
				}
				// if number of updates > 0, clear cache and then display an update message.
				if ($updated)
					$this->display_message(sprintf(__('%s series were updated successfully !', $this->textdomain),$updated), 'updated');
			}

			if (isset($_POST['eg_series_delete'])) {
				// check security
				check_admin_referer('eg_series_edit');

				$series = $_POST['series_to_delete'];

				$deleted = 0;
				if (isset($series) && sizeof($series)>0) {
					foreach ($series as $serie) {
						$deleted++;
						$sql = $wpdb->prepare('DELETE FROM '.$wpdb->postmeta.' WHERE meta_key="%s" AND meta_value="%s"', $meta_key, $serie);
						$wpdb->query($sql);
					}
					// if number of updates > 0, clear cache and then display an update message.
					if ($deleted)
						$this->display_message(sprintf(__('%s series were deleted successfully !', $this->textdomain),$deleted));
				}
			}
			// Refresh the list of series
			$values = $this->series_data->get_series_details($this->options['custom_field_name'], FALSE, FALSE, ($updated>0||$delete>0));

			echo '<div id="icon-tools" class="icon32"><br /></div>'.
					'<div class="wrap">'.
					'<h2>'.__('EG-Series Series Editor', $this->textdomain).'</h2>';
			if ( $values === FALSE || sizeof($values)==0 ) {
				_e('No serie available yet', $this->textdomain);
			}
			else {
				/* Display the page title */
				echo '<form name="post" id="post" method="post">'.
					wp_nonce_field('eg_series_edit').
					'<table class="widefat" cellspacing="0">'.
					'<thead>'.
					'<tr>'.
					'<th>&nbsp;</th>'.
					'<th class="manage-column">'.__('Series', $this->textdomain).'</th>'.
					'<th class="manage-column">'.__('New name', $this->textdomain).'</th>'.
					'</tr>'.
					'</thead>'.
					'<tbody>';

				// Display series list
				$class = 'alternate';
				foreach ($values as $serie_name => $serie_values) {
					echo '<tr>'.
						'<td><input type="checkbox" name="series_to_delete[]" value="'.$serie_name.'" /></td>'.
						'<td>'.$serie_name.' ('.sizeof($serie_values).__(' posts', $this->textdomain).')</td>'.
						'<td><input type="text" name="series['.$serie_name.']" /></td>'.
						'</tr>';
				}
				echo '</tbody></table>'.
					'<p class="submit">'.
					'<input class="button" type="submit" name="eg_series_rename" value="'.__('Update all &raquo;', $this->textdomain).'" /> '.
					'<input class="button" type="submit" name="eg_series_delete" value="'.__('Delete checked series', $this->textdomain).'" /></p>'.
					'</form>';
			}
			echo '</div>';
		} /* End of Edit_series */

		/**
		 * update_post_order
		 *
		 *
		 *
		 * @param 	$id			int		id of the post to modify
		 * @param 	$meta_key	string	meta key to use for series
		 * @param 	$order		int		order of the post in the serie
		 * @return 	none
		 */
		function update_post_order($id, $order) {
			global $wpdb;

			$sql = $wpdb->prepare('UPDATE '.$wpdb->posts.' SET menu_order=%u WHERE ID=%u', $order, $id );
			$wpdb->query($sql);
		} // End of update_post_order

	} /* End of class */
} /* End of class_exists */

$eg_series_admin = new EG_Series_Admin(EG_SERIES_PLUGIN_NAME,
										EG_SERIES_VERSION,
										EG_SERIES_CORE_FILE,
										EG_SERIES_OPTIONS_ENTRY,
										$EG_SERIES_DEFAULT_OPTIONS);

$eg_series_admin->set_textdomain(EG_SERIES_TEXTDOMAIN);
$eg_series_admin->set_stylesheets(FALSE, 'eg-series-admin.css');
$eg_series_admin->add_tinymce_button( 'EGSeries', 'tinymce');
$eg_series_admin->set_wp_versions('2.5', FALSE, FALSE, FALSE);
$eg_series_admin->load();

?>
